float Script : STANDARDSGLOBAL <
    string ScriptOutput = "color";
    string ScriptClass = "scene";
    string ScriptOrder = "postprocess";
> = 0.8;

float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;	
float Scale : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;

static float Si=Scale*0.1;
float LengthMax = 0.1;
static float BlurScale = Si*7;
float2 ViewportSize : VIEWPORTPIXELSIZE;

static float2 ViewportOffset = (float2(0.5,0.5)/ViewportSize);
static float2 SampStep = (float2(0.5,0.5) / ViewportSize)*2/1080*ViewportSize.y;

float3	LightPosition	: POSITION  < string Object = "Light"; >;
float3	LightDirection	: DIRECTION < string Object = "Light"; >;
float3	LightDiffuse	: DIFFUSE   < string Object = "Light"; >;
float3	LightAmbient	: AMBIENT   < string Object = "Light"; >;

float3   CameraPosition    : POSITION  < string Object = "Camera"; >;
float3   CameraDirection   : DIRECTION < string Object = "Camera"; >;
float4x4 WorldViewProjMatrix      : WORLDVIEWPROJECTION;
float4x4 ViewProjMatrix           : VIEWPROJECTION;

#define SAMPNUM 7

float ClearDepth  = 1.0;
float4 ClearColor = float4(0, 0, 0, 1);

texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    string Format = "D24S8";
>;

texture2D ScnMap : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
string Format = "A16B16G16R16F";
>;
sampler2D ScnSamp = sampler_state {
    texture = <ScnMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = CLAMP;
    AddressV = CLAMP;
};


////////////////////////////////////////////////////////////////////////////////////////////////
struct VS_OUTPUT {
	float4 Pos            : POSITION;
	float2 Tex            : TEXCOORD0;
};

VS_OUTPUT VS_passDraw( float4 Pos : POSITION, float4 Tex : TEXCOORD0 ) {
    VS_OUTPUT Out = (VS_OUTPUT)0; 
    
    Out.Pos = Pos;
    Out.Tex = Tex + ViewportOffset;
    return Out;
}

////////////////////////////////////////////////////////////////////////////////////////////////
float4 PS_passDraw(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(ScnSamp, Tex);
	float2 SampStepRate = SampStep;

	float3 ColorSampX0 = tex2D(ScnSamp, Tex + float2(-SampStepRate.x, 0.0)).rgb;
	float3 ColorSampY0 = tex2D(ScnSamp, Tex + float2(0.0, -SampStepRate.y)).rgb;
	float3 ColorSampY2 = tex2D(ScnSamp, Tex + float2(0.0, SampStepRate.y)).rgb;
	float3 ColorSampX2 = tex2D(ScnSamp, Tex + float2(SampStepRate.x, 0.0)).rgb;

	float XEdgeR = ColorSampX2.r - ColorSampX0.r;
	float YEdgeR = ColorSampY2.r - ColorSampY0.r;

	float XEdgeG = ColorSampX2.g - ColorSampX0.g;
	float YEdgeG = ColorSampY2.g - ColorSampY0.g;

	float XEdgeB = ColorSampX2.b - ColorSampX0.b;
	float YEdgeB = ColorSampY2.b - ColorSampY0.b;

	float EdgeRateLengthB = min(LengthMax, length(float2(YEdgeB, XEdgeB)));
	float EdgeRateLengthG = min(LengthMax, length(float2(YEdgeG, XEdgeG)));
	float EdgeRateLengthR = min(LengthMax, length(float2(YEdgeR, XEdgeR)));

	//水平強化
	EdgeRateLengthR *= max(0.0,min(10.0,(pow(YEdgeR/XEdgeR,2)+pow(XEdgeR/YEdgeR,2))*0.0002+0.3));
	EdgeRateLengthG *= max(0.0,min(10.0,(pow(YEdgeG/XEdgeG,2)+pow(XEdgeG/YEdgeG,2))*0.0002+0.3));
	EdgeRateLengthB *= max(0.0,min(10.0,(pow(YEdgeB/XEdgeB,2)+pow(XEdgeB/YEdgeB,2))*0.0002+0.3));

	float2 EdgeRateR = normalize(float2(-YEdgeR, XEdgeR))*EdgeRateLengthR;
	float2 EdgeRateG = normalize(float2(-YEdgeG, XEdgeG))*EdgeRateLengthG;
	float2 EdgeRateB = normalize(float2(-YEdgeB, XEdgeB))*EdgeRateLengthB;





	float2 SampStepRateR=SampStep*EdgeRateR*BlurScale;
	float BlurColor = 0.0;
	float e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateR;
		BlurColor += tex2D(ScnSamp, Tex + SampStepTex).r * e;
		n += e;
	}
	Color.r = BlurColor / n;

	float2 SampStepRateG = SampStep*EdgeRateG*BlurScale;
	BlurColor = 0.0;
	e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateG;
		BlurColor += tex2D(ScnSamp, Tex + SampStepTex).g * e;
		n += e;
	}
	Color.g = BlurColor / n;

	float2 SampStepRateB = SampStep*EdgeRateB*BlurScale;
	BlurColor = 0.0;
	e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2); 
		float2 SampStepTex = i*SampStepRateB;
		BlurColor += tex2D(ScnSamp, Tex + SampStepTex).b * e;
		n += e;
	}
	Color.b = BlurColor / n;


		return Color;
}

////////////////////////////////////////////////////////////////////////////////////////////////
technique AA <
	string Script =

	"RenderColorTarget0=ScnMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"ScriptExternal=Color;"

	"RenderColorTarget0=;"
	"RenderDepthStencilTarget=;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw;"
	;


> {
	pass Draw < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////
