float Script : STANDARDSGLOBAL <
    string ScriptOutput = "color";
    string ScriptClass = "scene";
    string ScriptOrder = "postprocess";
> = 0.8;

float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;	
float Scale : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;

static float Si=Scale*0.1;
float LengthMax = 1.0;
static float BlurScale = Si;
static float BlurScale1 = 0.5;
static float BlurScale2 = Si;
float2 ViewportSize : VIEWPORTPIXELSIZE;

static float2 ViewportOffset = (float2(0.5,0.5)/ViewportSize);
static float2 SampStep = (float2(0.5,0.5) / ViewportSize)*2;

float3	LightPosition	: POSITION  < string Object = "Light"; >;
float3	LightDirection	: DIRECTION < string Object = "Light"; >;
float3	LightDiffuse	: DIFFUSE   < string Object = "Light"; >;
float3	LightAmbient	: AMBIENT   < string Object = "Light"; >;

float3   CameraPosition    : POSITION  < string Object = "Camera"; >;
float3   CameraDirection   : DIRECTION < string Object = "Camera"; >;
float4x4 WorldViewProjMatrix      : WORLDVIEWPROJECTION;
float4x4 ViewProjMatrix           : VIEWPROJECTION;
float4x4 ViewMatrix      : VIEW;
static float3x3 VMat = 
{ { ViewMatrix._11, ViewMatrix._12, ViewMatrix._13 },
{ ViewMatrix._21, ViewMatrix._22, ViewMatrix._23 },
{ ViewMatrix._31, ViewMatrix._32, ViewMatrix._33 },
};

#define SAMPNUM 16

float ClearDepth  = 1.0;
float4 ClearColor = float4(0, 0, 0, 1);


shared texture2D NormalDepthKrMap : RENDERCOLORTARGET ;
sampler2D NormalDepthSamp = sampler_state {
	texture = <NormalDepthKrMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};



texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    string Format = "D24S8";
>;

texture2D ScnMap : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
    string Format = "A8B8G8R8" ;
>;
sampler2D ScnSamp = sampler_state {
    texture = <ScnMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};


texture2D Scn2Map : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
    string Format = "A8B8G8R8" ;
>;
sampler2D Scn2Samp = sampler_state {
    texture = <Scn2Map>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};


texture2D Scn3Map : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
    string Format = "A8B8G8R8" ;
>;
sampler2D Scn3Samp = sampler_state {
    texture = <Scn3Map>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};

texture2D Scn4Map : RENDERCOLORTARGET <
    float2 ViewPortRatio = {1.0,1.0};
    int MipLevels = 1;
    string Format = "A8B8G8R8" ;
>;
sampler2D Scn4Samp = sampler_state {
    texture = <Scn4Map>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = MIRROR;
    AddressV = MIRROR;
};



////////////////////////////////////////////////////////////////////////////////////////////////
struct VS_OUTPUT {
	float4 Pos            : POSITION;
	float2 Tex            : TEXCOORD0;
};

VS_OUTPUT VS_passDraw( float4 Pos : POSITION, float4 Tex : TEXCOORD0 ) {
    VS_OUTPUT Out = (VS_OUTPUT)0; 
    
    Out.Pos = Pos;
    Out.Tex = Tex + ViewportOffset;
    return Out;
}




float4 PS_passDraw0(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(ScnSamp, Tex);
	float2 SampStepRate = SampStep;
	float4 ND=tex2D(NormalDepthSamp,Tex).xyzw;

	//float4 ColorSampX0D = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	//float4 ColorSampY0D = tex2D(NormalDepthSamp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	//float4 ColorSampY2D = tex2D(NormalDepthSamp, Tex + float2(0.0, SampStepRate.y)).rgba;
	//float4 ColorSampX2D = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	//float GapMax=max(max(max(pow(ND.a-ColorSampX0D.a,2),pow(ND.a-ColorSampY0D.a,2)),
	//pow(ND.a-ColorSampX2D.a,2)),
	//pow(ND.a-ColorSampY2D.a,2));
	
	
	float4 ColorSampX0D = tex2D(ScnSamp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	float4 ColorSampY0D = tex2D(ScnSamp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	float4 ColorSampY2D = tex2D(ScnSamp, Tex + float2(0.0, SampStepRate.y)).rgba;
	float4 ColorSampX2D = tex2D(ScnSamp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	float GapMax=max(max(max(distance(ColorSampX0D.rgb,Color.rgb),distance(ColorSampX2D.rgb,Color.rgb)),
	distance(ColorSampY0D.rgb,Color.rgb)),distance(ColorSampY2D.rgb,Color.rgb));
	
	float3 VNormal=normalize(mul(ND.xyzw,VMat));
	float2 EdgeRate=normalize(float2(VNormal.y,VNormal.x))*BlurScale2* min(0.05,pow(GapMax,2));
	
	
	float2 SampStepRateR = SampStep*EdgeRate;
	float3 BlurColor = 0.0;
	float e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateR;
		BlurColor += tex2D(ScnSamp, Tex + SampStepTex).rgb * e;
		n += e;
	}
	Color.rgb = BlurColor / n;
	//Color.rgb = GapMax;


	return Color;
}
float4 PS_passDraw1(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(Scn2Samp, Tex);
	float2 SampStepRate = SampStep;
	float4 ND=tex2D(NormalDepthSamp,Tex).xyzw;

	//float4 ColorSampX0D = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	//float4 ColorSampY0D = tex2D(NormalDepthSamp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	//float4 ColorSampY2D = tex2D(NormalDepthSamp, Tex + float2(0.0, SampStepRate.y)).rgba;
	//float4 ColorSampX2D = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	//float GapMax=max(max(max(pow(ND.a-ColorSampX0D.a,2),pow(ND.a-ColorSampY0D.a,2)),
	//pow(ND.a-ColorSampX2D.a,2)),
	//pow(ND.a-ColorSampY2D.a,2));
	
	
	float4 ColorSampX0D = tex2D(Scn2Samp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	float4 ColorSampY0D = tex2D(Scn2Samp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	float4 ColorSampY2D = tex2D(Scn2Samp, Tex + float2(0.0, SampStepRate.y)).rgba;
	float4 ColorSampX2D = tex2D(Scn2Samp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	float GapMax=max(max(max(distance(ColorSampX0D.rgb,Color.rgb),distance(ColorSampX2D.rgb,Color.rgb)),
	distance(ColorSampY0D.rgb,Color.rgb)),distance(ColorSampY2D.rgb,Color.rgb));
	
	float3 VNormal=normalize(mul(ND.xyzw,VMat));
	float2 EdgeRate=normalize(float2(VNormal.y,VNormal.x))*BlurScale2* min(0.05,pow(GapMax,2));
	
	
	float2 SampStepRateR = SampStep*EdgeRate;
	float3 BlurColor = 0.0;
	float e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateR;
		BlurColor += tex2D(Scn2Samp, Tex + SampStepTex).rgb * e;
		n += e;
	}
	Color.rgb = BlurColor / n;
	//Color.rgb = GapMax;


	return Color;
}
float4 PS_passDraw2(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(Scn3Samp, Tex);
	float2 SampStepRate = SampStep;
	float4 ND=tex2D(NormalDepthSamp,Tex).xyzw;

	//float4 ColorSampX0D = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	//float4 ColorSampY0D = tex2D(NormalDepthSamp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	//float4 ColorSampY2D = tex2D(NormalDepthSamp, Tex + float2(0.0, SampStepRate.y)).rgba;
	//float4 ColorSampX2D = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	//float GapMax=max(max(max(pow(ND.a-ColorSampX0D.a,2),pow(ND.a-ColorSampY0D.a,2)),
	//pow(ND.a-ColorSampX2D.a,2)),
	//pow(ND.a-ColorSampY2D.a,2));
	
	
	float4 ColorSampX0D = tex2D(Scn3Samp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	float4 ColorSampY0D = tex2D(Scn3Samp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	float4 ColorSampY2D = tex2D(Scn3Samp, Tex + float2(0.0, SampStepRate.y)).rgba;
	float4 ColorSampX2D = tex2D(Scn3Samp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	float GapMax=max(max(max(distance(ColorSampX0D.rgb,Color.rgb),distance(ColorSampX2D.rgb,Color.rgb)),
	distance(ColorSampY0D.rgb,Color.rgb)),distance(ColorSampY2D.rgb,Color.rgb));
	
	float3 VNormal=normalize(mul(ND.xyzw,VMat));
	float2 EdgeRate=normalize(float2(VNormal.y,VNormal.x))*BlurScale2* min(0.05,pow(GapMax,2));
	
	
	float2 SampStepRateR = SampStep*EdgeRate;
	float3 BlurColor = 0.0;
	float e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateR;
		BlurColor += tex2D(Scn3Samp, Tex + SampStepTex).rgb * e;
		n += e;
	}
	Color.rgb = BlurColor / n;
	//Color.rgb = GapMax;


	return Color;
}

float4 PS_passDraw3(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = tex2D(Scn4Samp, Tex);
	float2 SampStepRate = SampStep;
	float4 ND=tex2D(NormalDepthSamp,Tex).xyzw;

	//float4 ColorSampX0D = tex2D(NormalDepthSamp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	//float4 ColorSampY0D = tex2D(NormalDepthSamp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	//float4 ColorSampY2D = tex2D(NormalDepthSamp, Tex + float2(0.0, SampStepRate.y)).rgba;
	//float4 ColorSampX2D = tex2D(NormalDepthSamp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	//float GapMax=max(max(max(pow(ND.a-ColorSampX0D.a,2),pow(ND.a-ColorSampY0D.a,2)),
	//pow(ND.a-ColorSampX2D.a,2)),
	//pow(ND.a-ColorSampY2D.a,2));
	
	
	float4 ColorSampX0D = tex2D(Scn4Samp, Tex + float2(-SampStepRate.x, 0.0)).rgba;
	float4 ColorSampY0D = tex2D(Scn4Samp, Tex + float2(0.0, -SampStepRate.y)).rgba;
	float4 ColorSampY2D = tex2D(Scn4Samp, Tex + float2(0.0, SampStepRate.y)).rgba;
	float4 ColorSampX2D = tex2D(Scn4Samp, Tex + float2(SampStepRate.x, 0.0)).rgba;
	
	
	float GapMax=max(max(max(distance(ColorSampX0D.rgb,Color.rgb),distance(ColorSampX2D.rgb,Color.rgb)),
	distance(ColorSampY0D.rgb,Color.rgb)),distance(ColorSampY2D.rgb,Color.rgb));
	
	
	float3 VNormal=normalize(mul(ND.xyzw,VMat));
	float2 EdgeRate=normalize(float2(VNormal.y,VNormal.x))*BlurScale2* min(0.05,pow(GapMax,2));
	
	
	float2 SampStepRateR = SampStep*EdgeRate;
	float3 BlurColor = 0.0;
	float e, n = 0;
	[unroll]
	for (int i = -SAMPNUM; i <= SAMPNUM; i += 1) {
		e = exp(-pow((float)i / (SAMPNUM / 2.0), 2) / 2);
		float2 SampStepTex = i*SampStepRateR;
		BlurColor += tex2D(Scn4Samp, Tex + SampStepTex).rgb * e;
		n += e;
	}
	Color.rgb = BlurColor / n;
	//Color.rgb = GapMax;


	return Color;
}
////////////////////////////////////////////////////////////////////////////////////////////////
technique AA <
	string Script =

	"RenderColorTarget0=ScnMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"ScriptExternal=Color;"

	"RenderColorTarget0=Scn2Map;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw0;"
	
	"RenderColorTarget0=Scn3Map;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw1;"
	
	"RenderColorTarget0=Scn4Map;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw2;"
	
	"RenderColorTarget0=;"
	"RenderDepthStencilTarget=;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=Draw3;"
	;


> {
	pass Draw0 < string Script = "Draw=Buffer;"; > {
		//AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw0();
	}
	pass Draw1 < string Script = "Draw=Buffer;"; > {
		//AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw1();
	}
	pass Draw2 < string Script = "Draw=Buffer;"; > {
		//AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw2();
	}
	pass Draw3 < string Script = "Draw=Buffer;"; > {
		//AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDraw3();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////
