///////////////////////////////////////////////////////////////////////////////////
//KrCanvas
//by
///////////////////////////////////////////////////////////////////////////////////
float Script : STANDARDSGLOBAL <
	string ScriptOutput = "color";
	string ScriptClass = "scene";
	string ScriptOrder = "postprocess";
> = 0.8;


float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;
float Si10 : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;
float X : CONTROLOBJECT < string name = "(self)"; string item = "X"; >;
float Y : CONTROLOBJECT < string name = "(self)"; string item = "Y"; >;
float Z : CONTROLOBJECT < string name = "(self)"; string item = "Z"; >;
float Rx : CONTROLOBJECT < string name = "(self)"; string item = "Rx"; >;
float Ry : CONTROLOBJECT < string name = "(self)"; string item = "Ry"; >;
float Rz : CONTROLOBJECT < string name = "(self)"; string item = "Rz"; >;


float3   CameraPosition    : POSITION  < string Object = "Camera"; >;
float3   CameraDirection    : DIRECTION  < string Object = "Camera"; >;




float2 ViewportSize : VIEWPORTPIXELSIZE;
static float2 ViewportOffset = (float2(0.5,0.5)/ViewportSize)/2;
static float2 SampStep =  (float2(0.5, 0.5) / ViewportSize);
float4x4 ProjMatrix		: PROJECTION;
float4x4 ViewProjMatrix      : VIEWPROJECTION;
float4x4 InvViewMatrix  	: VIEWINVERSE;
float4x4 WorldMatrix     : WORLD;
float4x4 ViewMatrix      : VIEW;
static float3 DOFPos = WorldMatrix._41_42_43;
static float2 scaleProj = float2(1.0 / ProjMatrix._11_22);


static float DOFZ= dot(CameraDirection,DOFPos-CameraPosition);

static float BackRate=1.0/DOFZ*Si10;

static float GradScale =0.1*(1.0+Rx * 180 / 3.14);


#define SAMPNUM 96
#define SAMPNUM2 96
#define SAMPNUM3 16
#define SAMPNUM3B 33
static float2 SampStep2 =  (float2(0.5, 0.5) / ViewportSize)*ViewportSize.y/10/SAMPNUM3B*BackRate;


float ClearDepth = 1.0;
float4 ClearColor= float4(0,0,0,0);

shared texture2D NormalDepthKrMap : RENDERCOLORTARGET ;
sampler2D NormalDepthSamp = sampler_state {
	texture = <NormalDepthKrMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

shared texture2D WPosKrMap : RENDERCOLORTARGET ;
sampler2D WPosSamp = sampler_state {
	texture = <WPosKrMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET <
	float2 ViewPortRatio = { 2.0, 2.0 };
	string Format = "D24S8";
>;

texture2D ScnMap : RENDERCOLORTARGET <
	float2 ViewPortRatio = { 1.0, 1.0 };
	int MipLevels = 1;
	string Format = "A16B16G16R16F";
>;
sampler2D ScnSamp = sampler_state {
	texture = <ScnMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

texture BokehMaskTex <
	string ResourceName = "Hex2.png";
	int Miplevels = 1;
>;
sampler BokehMask = sampler_state {
	texture = <BokehMaskTex>;
	MinFilter = NONE;
	MagFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

////////////////////////////////////////////////////////////////////////////////////////////////
struct VS_OUTPUT {
	float4 Pos            : POSITION;
	float2 Tex            : TEXCOORD0;
};

VS_OUTPUT VS_passDraw(float4 Pos : POSITION, float4 Tex : TEXCOORD0) {
	VS_OUTPUT Out = (VS_OUTPUT)0;

	Out.Pos = Pos;
	Out.Tex = Tex + ViewportOffset;
	return Out;
}

////////////////////////////////////////////////////////////////////////////////////////////////
float4 PS_passDrawB(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = (tex2D(ScnSamp, Tex));
	float4 ND = tex2D(NormalDepthSamp, Tex);
	float3 Normal = ND.rgb;
	float Depth = ND.a;
	float4 WPos =tex2D(WPosSamp,Tex);
	

	float BackRate= saturate((Depth-DOFZ)*GradScale);
	
	float2 STexSampA = SampStep2*BackRate;
	float3 BColor = tex2D(ScnSamp, Tex).rgb;
	float BokeRateT=1.0;

	[unroll]
	for (int ix =-SAMPNUM3; ix <= SAMPNUM3; ix++) {
	for (int iy = -SAMPNUM3; iy <= SAMPNUM3; iy++) {
		float2 AddTexB = float2(ix,iy);
		float2 Stex = Tex + STexSampA*AddTexB;
	

		float DepthS = tex2D(NormalDepthSamp, Stex).a;
		float BokeDepthRate = saturate((DepthS-DOFZ)*GradScale);
		float BokeTexRate =tex2D(BokehMask,AddTexB/(SAMPNUM3B)+0.5).r;
		float BokeRate =BokeTexRate*BokeDepthRate;
		float3 BColorS = tex2D(ScnSamp, Stex).rgb;

		BColor += BColorS*BokeRate;
		BokeRateT+=BokeRate;

	}
	}
	BColor.rgb/=BokeRateT;
	
	Color.rgb=BColor;
	
		
	return Color;
}
////////////////////////////////////////////////////////////////////////////////////////////////

technique FILTER <
	string Script =

	"RenderColorTarget0=ScnMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"ScriptExternal=Color;"
	

	"RenderColorTarget0=;"
	"RenderDepthStencilTarget=;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=DrawB;"
	
	;

> {

	pass DrawB < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		AlphaTestEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDrawB();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////
