///////////////////////////////////////////////////////////////////////////////////
//KrCanvas
//by
///////////////////////////////////////////////////////////////////////////////////
float Script : STANDARDSGLOBAL <
	string ScriptOutput = "color";
	string ScriptClass = "scene";
	string ScriptOrder = "postprocess";
> = 0.8;


float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;
float Si10 : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;
float X : CONTROLOBJECT < string name = "(self)"; string item = "X"; >;
float Y : CONTROLOBJECT < string name = "(self)"; string item = "Y"; >;
float Z : CONTROLOBJECT < string name = "(self)"; string item = "Z"; >;
float Rx : CONTROLOBJECT < string name = "(self)"; string item = "Rx"; >;
float Ry : CONTROLOBJECT < string name = "(self)"; string item = "Ry"; >;
float Rz : CONTROLOBJECT < string name = "(self)"; string item = "Rz"; >;


float3   CameraPosition    : POSITION  < string Object = "Camera"; >;
float3   CameraDirection    : DIRECTION  < string Object = "Camera"; >;


float time : Time;
static float timeTex=fmod(round(time*6)*0.1,1.0);

float2 ViewportSize : VIEWPORTPIXELSIZE;
static float2 ViewportOffset = (float2(0.5,0.5)/ViewportSize);
static float2 SampStep =  (float2(0.5, 0.5) / ViewportSize);
float4x4 ProjMatrix		: PROJECTION;
float4x4 ViewProjMatrix      : VIEWPROJECTION;
float4x4 InvViewMatrix  	: VIEWINVERSE;
float4x4 WorldMatrix     : WORLD;
float4x4 ViewMatrix      : VIEW;
static float3 DOFPos = WorldMatrix._41_42_43;
static float2 scaleProj = float2(1.0 / ProjMatrix._11_22);



static float DOFZ= dot(CameraDirection,DOFPos-CameraPosition);

static float FrontRate=DOFZ*Si10*0.001;
static float GradSize =10*(1.0+Rx * 180 / 3.14);
static float GradScale=0.1*(1.0+Ry * 180 / 3.14);

#define SAMPNUM 96
#define SAMPNUM2 96
#define SAMPNUM3 16
#define SAMPNUM3B 33
static float2 SampStep2 =  (float2(0.5, 0.5) / ViewportSize)*ViewportSize.y/10/SAMPNUM3B*FrontRate;


float ClearDepth = 1.0;
float4 ClearColor= float4(0,0,0,0);

shared texture2D NormalDepthKrMap : RENDERCOLORTARGET ;
sampler2D NormalDepthSamp = sampler_state {
	texture = <NormalDepthKrMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

shared texture2D WPosKrMap : RENDERCOLORTARGET ;
sampler2D WPosSamp = sampler_state {
	texture = <WPosKrMap>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET <
	float2 ViewPortRatio = { 1.0, 1.0 };
	string Format = "D24S8";
>;

texture2D ScnMap : RENDERCOLORTARGET <
	float2 ViewPortRatio = { 1.0, 1.0 };
	int MipLevels = 1;
	string Format = "A16B16G16R16F";
>;
sampler2D ScnSamp = sampler_state {
	texture = <ScnMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
};
texture2D FMMap : RENDERCOLORTARGET <
	float2 ViewPortRatio = { 1.0,1.0 };
int MipLevels = 1;
string Format = "A16B16G16R16F";
>;
sampler2D FMSamp = sampler_state {
	texture = <FMMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
};
texture BokehMaskTex <
	string ResourceName = "Hex2.png";
	int Miplevels = 1;
>;
sampler BokehMask = sampler_state {
	texture = <BokehMaskTex>;
	MinFilter = NONE;
	MagFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

////////////////////////////////////////////////////////////////////////////////////////////////
struct VS_OUTPUT {
	float4 Pos            : POSITION;
	float2 Tex            : TEXCOORD0;
};

VS_OUTPUT VS_passDraw(float4 Pos : POSITION, float4 Tex : TEXCOORD0) {
	VS_OUTPUT Out = (VS_OUTPUT)0;

	Out.Pos = Pos;
	Out.Tex = Tex + ViewportOffset;
	return Out;
}

////////////////////////////////////////////////////////////////////////////////////////////////
float4 PS_passDrawFM(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = (tex2D(ScnSamp, Tex));
	float4 ND = tex2D(NormalDepthSamp, Tex);
	float3 Normal = ND.rgb;
	float Depth = ND.a;
	float4 WPos =tex2D(WPosSamp,Tex);
	float3 WPosS =  WPos * 0.01;

	float DOFW= Depth;
	Color.rgb=Color.rgb+normalize(Color.rgb)*pow(max(0.0,length(Color.rgb)-0.8),2)*max(0.0,FrontRate-1.0);
	Color.a*=saturate((DOFZ-DOFW-GradSize)*GradScale);
	return Color;
}
////////////////////////////////////////////////////////////////////////////////////////////////
float4 PS_passDrawC(float2 Tex: TEXCOORD0) : COLOR
{
	float4 Color = (tex2D(ScnSamp, Tex));
	float4 ND = tex2D(NormalDepthSamp, Tex);
	float3 Normal = ND.rgb;
	float Depth = ND.a;
	float4 WPos =tex2D(WPosSamp,Tex);
	float3 WPosS =  WPos * 0.01;
	
	
	
	float2 stexAdd = 0;
	
	float2 STexSampA = SampStep2;
	
	
	float4 FColor =tex2D(FMSamp, Tex).rgba*tex2D(FMSamp, Tex).a;
	float BokeRateT=1.0;
	
	
	[unroll]
	for (int ix =-SAMPNUM3; ix <= SAMPNUM3; ix++) {
	for (int iy = -SAMPNUM3; iy <= SAMPNUM3; iy++) {
		float2 AddTexB = float2(ix,iy);
		float2 Stex = Tex + STexSampA*AddTexB;
	
		float BokeRate =tex2D(BokehMask,AddTexB/(SAMPNUM3B)+0.5).r;
		FColor += tex2D(FMSamp, Stex).rgba*BokeRate;
		BokeRateT+=BokeRate;

	}
	}
	
	
	FColor.rgba/=BokeRateT;
	//FColor.rgb/=FColor.a;
	
	
	Color.rgb=lerp(Color.rgb,FColor.rgb,saturate(FColor.a*2));
	
	
	
	
	
	
	
	
	
	
	
	
	return Color;
}

////////////////////////////////////////////////////////////////////////////////////////////////

technique FILTER <
	string Script =

	"RenderColorTarget0=ScnMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"ScriptExternal=Color;"

	
	"RenderColorTarget0=FMMap;"
	"RenderDepthStencilTarget=DepthBuffer;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=DrawFM;"
	
	
	"RenderColorTarget0=;"
	"RenderDepthStencilTarget=;"
	"ClearSetColor=ClearColor;"
	"ClearSetDepth=ClearDepth;"
	"Clear=Color;"
	"Clear=Depth;"
	"Pass=DrawC;"
	;

> {
	pass DrawFM < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		AlphaTestEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDrawFM();
	}
	pass DrawC < string Script = "Draw=Buffer;"; > {
		AlphaBlendEnable = FALSE;
		AlphaTestEnable = FALSE;
		VertexShader = compile vs_3_0 VS_passDraw();
		PixelShader = compile ps_3_0 PS_passDrawC();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////
